/**
 * A light string operator with function supported by Lucien
 * eg:
 *    si.solve('${xxx}',obj);
 *    si.solve('${xxx}',arg1,arg2,...);
 *    si.solve('${xxx|functionName}');
 *    si.solve('${xxx|functionName(arg1,arg2,...)}');
 */
(function () {
  //analyse environment : Browser or Node
  const root = (typeof window === 'object' && window.window === window && window) ||
    (typeof global === 'object' && global.global === global && global) || this || {};
  const si = root.si = {};
  if (typeof module === 'object' && typeof module.exports === 'object') {
    module.exports = root.si;
  }
  /**
   * 添加自定义方法
   * @param {string} fName 方法名
   * @param {function} fFunc 方法体
   */
  si.inFunction = (fName, fFunc) => {
    if (typeof fName !== 'string' || typeof fFunc !== 'function') {
      console.error('invalid function!');
      return false;
    }
    if (!si.$$customizeFunctions) {
      si.$$customizeFunctions = {};//reserve customize functions
    }
    if (si.$$customizeFunctions[fName]) {
      console.error('Function ' + fName + ' is already exist!');
      return false;
    }
    si.$$customizeFunctions[fName] = fFunc;
  };
  /**
   * 移除自定义方法
   * @param {string} fName 待移除的方法名
   */
  si.unFunction = fName => {
    try {
      si.$$customizeFunctions[fName] = null;
      delete si.$$customizeFunctions[fName];
      return true;
    } catch (e) {
      return false;
    }
  };
  /**
   * 获取已经注册的方法，优先查找自定义方法
   * @param {string} fName 方法名
   */
  si.getFunction = fName => {
    return si.$$customizeFunctions[fName] || String.prototype[fName];
  };
  /**
  * 格式化操作
  * @param {*} strInput 待格式化的字符串
  * @param {*} fName 方法名:functionName 或方法调用: functionName(arg1,arg2,...)
  */
  si.handleFormat = (strInput, fName) => {
    var arr = fName.split(/[(),]/);
    var _name = arr[0];
    arr.shift();
    arr.pop();
    var func = si.getFunction(_name);
    if (func) {
      return func.apply(strInput, arr);
    } else {
      console.warn('Function ' + _name + ' cannot be found, original value ' + strInput + ' will be returned.');
      return strInput;
    }
  };
  /**
  * 字符串解析模板
  * @param {string} str 待填充内容格式为${...}的字符串
  * @param {*} args 参数，single object or several values
  */
  si.solve = (str, ...args) => {
    if (typeof str === 'undefined' || str === '') {
      console.error('Parameter 0 should have truly content!');
      return false;
    }
    let result = str;//输出结果
    const len = args.length;
    // const reg = new RegExp('(?<=\${)[^\${}]*(?=})','g');
    // const reg = /(?<=\${)[^\${}]*(?=})/g;
    const reg = /\${[^${}]*}/g;

    if (len === 1 && typeof args[0] === 'object') {
      // 将对象的key-value依次传入
      result = str.replace(reg, matcher => {
        let m = matcher.slice(2, -1);//remove ${}
        let tmp = m.replace(/\s/g, '').split('|'),
          key = tmp[0],
          fun = tmp[1];
        let keyArray = key.split('.');
        let val = args[0];
        for (let k in keyArray) { // 解析对象形式的字符串 ${a.b.c}
          val = val[keyArray[k]]
        }
        if (val) {
          if (fun) {//过滤器函数
            return si.handleFormat(val, fun);
          }
          return val;
        }
        return "";//没有对应的匹配项则返回空字符串
      });
    } else {
      // 将参数值依次传入
      let i = 0;
      result = str.replace(reg, matcher => {
        let m = matcher.slice(2, -1);
        let tmp = m.replace(/\s/g, '').split('|'),
          key = tmp[0],
          fun = tmp[1],
          ag = args[i++];
        if (ag) {
          let val = ag;
          if(typeof val === 'object'){
            let keyArray = key.split('.');
            for (let k in keyArray) { // 解析对象形式的字符串 ${a.b.c}
              val = val[keyArray[k]]
            }
          }
          if (fun) {//过滤器函数
            return si.handleFormat(val, fun);
          }
          return val;
        }
        return "";//没有对应的匹配项则返回空字符串
      })
    }
    return result;
  };
})();